#!/bin/bash
##############################################################################
# audioMMT
# Software to extract and convert MeTaLMaDNeSS.TeaM releases to several other
# formats.
#
# Returns:
# 0 - Everything went OK
# 1 - Failed to detect release type, probably a bug in audiommt.sh
# 2 - There was no preferences file, default created
# 3 - Invalid operation mode requested, usage displayed
# 4 - Error during operation
#
# It requires the following software. The number in parentheses is the 
# software version used for testing. Different versions might (or not) work.
# Particularly, older versions of shntools will not work properly:
#
# bash:     For executing the commands in the script. (3.2.9)
# wodim:    For recording the audio CD. (wodim 1.1.6)
# cuetools: For processing audio files. 
#           Get it from http://developer.berlios.de/projects/cuetools/
# flac:     For compressing/uncompressing FLAC files. (1.1.4)
# id3:      For tagging the resulting MP3 files. (0.78)
#           Get it from http://home.wanadoo.nl/squell/id3.html
# lame:     For compressing on MP3. (3.97)
# mac:      For compressing/uncompresing APE files. (3.99u4b5)
# mppenc:   For compressing on MPC. (1.16)
#						Get it from http://www.musepack.net
# oggenc:   For compressing on OGG. (1.1.1)
# sed:      It works with the GNU one, not tested with any other. (4.1.5)
# shntool:  For processing audio files. (3.0.3)
#           Get it from http://freshmeat.net/projects/shntool/
# unrar:    For uncompressing RAR releases (3.70 beta 1)
#						Get it from http://www.rarsoft.com
# unzip:    For uncompressing ZIP.WV releases (5.52)
# wavpack:  For compressing WV files (4.41.0)
# wvunpack: For uncompressing WV files (4.41.0)
##############################################################################

##############################################################################
# cleanup function
#
# It deletes the temporary folder used during conversion.
##############################################################################
cleanup ()
{
	echo ""
	echo "Cleaning up temporary files"
	cd "${TEMP_FOLDER}"/..
	rm -rf "${TEMP_FOLDER}"
} 

##############################################################################
# usage function
#
# It shows the usage instructions
##############################################################################
usage ()
{
	echo " "
	echo "`basename $0`"
	echo "Usage: `basename $0` <MODE> <MMT_FILE> [<MMT_FILE>*]"
	echo " "
	echo "<MODE> Should be one of:"
	echo "   -mp3     : Use it to convert the release to individual mp3 files"
	echo "   -mpc     : Use it to convert the release to individual mpc files"
	echo "   -ogg     : Use it to convert the release to individual ogg files"
	echo "   -ape     : Use it to convert the release to individual ape files"
	echo "   -flac    : Use it to convert the release to individual flac files"
	echo "   -wv      : Use it to convert the release to individual wv files"
	echo "   -mp3cue  : Use it to convert the release to mp3 file + cue"
	echo "   -mpccue  : Use it to convert the release to mpc file + cue"
	echo "   -oggcue  : Use it to convert the release to ogg file + cue"
	echo "   -apecue  : Use it to convert the release to ape file + cue"
	echo "   -flaccue : Use it to convert the release to flac file + cue"
	echo "   -wvcue   : Use it to convert the release to wv file + cue"
	echo "   -burn    : Use it to burn the release as an audio CD"
	echo "   -extract : Use it to uncompress the release to the temp folder"
	echo "<MMT_FILE>  : RAR or ZIP file containing a MMT release."
	echo "              It can also be a folder with an uncompressed release"
	echo "              There can be as many as needed, all will be processed"
	echo "              the same way"
	echo " "
}

##############################################################################
# moveExtraFiles function
#
# It moves the extra files (like rip log, scans, etc) to the destination folder
##############################################################################
moveExtraFiles ()
{
	echo ""
	echo "Moving extra files to $DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR"
	mv * "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR" &>/dev/null
}

##############################################################################
# releasetype function
#
# It tells the script the kind of release it's dealing with.
#
# Release types are:
# 1 - One image, one CUE file (normal release)
# 2 - One image, more than once CUE file
# 3 - More than one image, one CUE file per image (normal release, various CDs)
# 4 - More than one image, more than one CUE file per image
# 5 - Separated tracks release (no CUE)
# 6 - Other (?)
##############################################################################
releasetype ()
{
	# Count the number of cue files found
	NUMBER_OF_CUES=`ls *.cue 2>/dev/null | grep -c .`

	# Count the total number of audio files	
	NUMBER_OF_IMAGES=`ls *.ape *.flac *.wv *.wav 2>/dev/null | grep -c .`
	
	if [[ $NUMBER_OF_CUES -eq $NUMBER_OF_IMAGES && $NUMBER_OF_CUES -eq 1 ]]; then #Standard 1CD releases
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 1"
		return 1
	elif [[ $NUMBER_OF_CUES -eq $NUMBER_OF_IMAGES && $NUMBER_OF_CUES -gt 1 ]]; then #Standard multiCD releases
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 3"
		return 3
	elif [[ $NUMBER_OF_CUES -gt $NUMBER_OF_IMAGES && $NUMBER_OF_IMAGES -eq 1 ]]; then #1CD Release, several cues
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 2"
		return 2
	elif [[ $NUMBER_OF_CUES -gt $NUMBER_OF_IMAGES && $NUMBER_OF_IMAGES -gt 1 ]]; then #multiCD release, several cues per image
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 4"
		return 4
        elif [[ $NUMBER_OF_IMAGES -gt 0 ]]; then #It has less cues than audio files, probably a separated tracks release
                echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 5"
                return 5
	else #Unknown type of release, most probably not a valid release
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 6"
		return 6
	fi
}

##############################################################################
# compress function
#
# It compresses the supplied WAV file to the specified format. The WAV file is 
# deleted after compression
#
# First parameter should be the WAV filename to compress (with extension)
# Second parameter should be the destination filename (with path if needed)
# Third parameter should be the format it should be compressed to
# (one of ape,flac,mp3,mpc,wv)
##############################################################################
compress ()
{
	if [ $3 = "mp3" ]; then
		#Compress to MP3 the current WAV file
		lame $LAME_PRESET "$1" "$2"
	elif [ $3 = "mpc" ]; then
		#Compress to MPC the current WAV file
		mppenc $MPPENC_PRESET "$1" "$2"
	elif [ $3 = "ogg" ]; then
		#Compress to MPC the current WAV file
		oggenc $OGGENC_PRESET "$1" -o "$2"
	elif [ $3 = "ape" ]; then
		#Compress to APE the current WAV file
		mac "$1" "$2" $MAC_PRESET
	elif [ $3 = "flac" ]; then
		#Compress to FLAC the current WAV file
		flac $FLAC_PRESET "$1" "$2"
	elif [ $3 = "wv" ]; then
		#Compress to WV the current WAV file
		wavpack $WAVPACK_PRESET "$1" "$2"
	fi
	rm -f "$1"
}

##############################################################################
# tag function
#
# Tags an audio file with the supplied data (if possible)
# First parameter should be the file to tag
# Second parameter should be the TRACK_TITLE
# Third parameter should be the ALBUM_ARTIST
# Fourth parameter should be the ALBUM_TITLE
# Fifth parameter should be the ALBUM_YEAR
# Sixth parameter should be the TRACK_NUMBER
# Seventh parameter should be the file type (one of ape,flac,mp3,mpc,wv)
##############################################################################
tag ()
{
	if [ $7 = "mp3" ]; then
		#Tag the given MP3 file. It will fill ID3v1 and ID3v2 tags.
		id3 -12 -t "$2" -a "$3" -l "$4" -y "$5" -n "$6" "$1"
	elif [ $7 = "mpc" ]; then
		#Tag the given MPC file.
		echo "No tag yet for mpc files"
	elif [ $7 = "ogg" ]; then
		#Tag the given OGG file.
		vorbiscomment -a "$1" -t "TITLE=$2" -t "ARTIST=$3" -t "ALBUM=$4" -t "DATE=$5" -t "TRACKNUMBER=$6" 
	elif [ $7 = "ape" ]; then
		#Tag the given APE file.
		echo ""
	elif [ $7 = "flac" ]; then
		#Tag the given FLAC file.
		metaflac --set-tag="TITLE=$2" --set-tag="ARTIST=$3" --set-tag="ALBUM=$4" --set-tag="DATE=$5" --set-tag="TRACKNUMBER=$6" "$1"
	elif [ $7 = "wv" ]; then
		#Tag the given WV file.
		echo "No tag yet for wavpack files"
	fi
}

##############################################################################
# converttotracks function
#
# First parameter should be the CUE filename (with extension)
# Second parameter should be the IMAGE filename (no extension)
# Third parameter should be the extension of the IMAGE file (ape,flac,wv or wav)
# Fourth parameter should be the desired output extension
##############################################################################
converttotracks ()
{
	#Get album information and create destination folder
	ALBUM_TITLE=`cueprint -d %T "$1" | sed 's/\//_/g'`
	ALBUM_ARTIST=`cueprint -d %P "$1" | sed 's/\//_/g'`
	ALBUM_SUBDIR="$ALBUM_PREFIX$ALBUM_TITLE"
	ALBUM_YEAR="$YEAR"
	TRACKS=`cueprint -d %N "$1"`
	
	mkdir -p "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR"
	
	#Split the IMAGE in WAV tracks acording to the CUE and then 
	#delete the IMAGE and the CUE. Every track will be named "split-trackNN.wav"
	cuebreakpoints "$1" | shnsplit -n %02d -o wav "$2".$3

	#If the release was correctly split, continue
	if [ $? -eq 0 ]; then
		rm -f "$2".$3
	
		#Compress each of the WAVs resulting from splitting the IMAGE
		for I in `seq 1 $TRACKS`
		do
			TRACK_TITLE=`cueprint -t %t -n $I "$1" | sed 's/\//_/g'`
			if (( I < 10 )); then
				TRACK_NUMBER=0$I
			else
				TRACK_NUMBER=$I
			fi
		
			compress "split-track$TRACK_NUMBER.wav" "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR/$TRACK_NUMBER - $TRACK_TITLE".$4 $4
			tag "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR/$TRACK_NUMBER - $TRACK_TITLE".$4 "$TRACK_TITLE" "$ALBUM_ARTIST" "$ALBUM_TITLE" "$ALBUM_YEAR" "$TRACK_NUMBER" $4
			rm -f "split-track$TRACK_NUMBER.wav"
		done
		rm -f "$1"
	else
		cleanup
		error "Problem converting the release" 4
	fi
}

##############################################################################
# converttocue function
#
# First parameter should be the CUE filename (with extension)
# Second parameter should be the IMAGE filename (no extension)
# Third parameter should be the extension of the IMAGE file (ape,flac,wv or wav)
# Fourth parameter should be the desired output extension
##############################################################################
converttocue ()
{
	#Get album information and create destination folder
	ALBUM_TITLE=`cueprint -d %T "$1" | sed 's/\//_/g'`
	if [ x"$ALBUM_TITLE" = x"" ]; then
		ALBUM_TITLE="Unknown Album"
	fi
	ALBUM_ARTIST=`cueprint -d %P "$1" | sed 's/\//_/g'`
	if [ x"$ALBUM_ARTIST" = x"" ]; then
		ALBUM_ARTIST="Unknown Artist"
	fi
	ALBUM_SUBDIR="$ALBUM_PREFIX$ALBUM_TITLE"
	TRACKS=`cueprint -d %N $1`
	ALBUM_YEAR="$YEAR"
	
	#Create output folder
	mkdir -p "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR"

	# if the output format is different from the input one, convert the image file
	if [[ ! x"$3" = x"$4" ]]; then
		shnconv -o "$4" "$2"."$3"
		rm "$2"."$3"
	fi
	
	mv joined."$4" "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR/cdimage.$4"
	eval "sed 's/FILE \"""$2""."$3"\" WAVE/FILE \""cdimage"."$4"\" WAVE/' ""$1"" > temp_cdimage.cue"
	mv temp_cdimage.cue "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR/"cdimage.cue
}

##############################################################################
# converttocd function
#
# First parameter should be the CUE filename (with extension)
# Second parameter should be the IMAGE filename (no extension)
# Third parameter should be the extension of the IMAGE file (ape,flac or wv)
##############################################################################
converttocd ()
{
	#Uncompress the image to WAV and delete the original image
	shnconv -d "$TEMP_FOLDER" "$2".$3
	rm "$2".$3
		
	#Update the cue file with the new WAV extension
	eval "sed 's/FILE \"""$2""."$3"\" WAVE/FILE \"""$2"".wav\" WAVE/' ""$1"" > temp_cdimage.cue"
	rm "$1"
	mv temp_cdimage.cue "$1"
	
	#Burn the WAV file as an audio CD with CD-TEXT information :)
	cdrecord -v dev=1,0,0 -dao -driveropts=burnfree -eject -speed=8 -text -pad -cuefile="$1" -audio
}

##############################################################################
# process function
#
# First parameter should be the CUE filename (with extension)
# Second parameter should be the IMAGE filename (no extension)
# Third parameter should be the extension of the IMAGE file (ape,flac,wv or wav)
# Fourth parameter should be the conversion mode requested by the user
##############################################################################
process ()
{
	if [[ $4 = "mp3" || $4 = "mpc" || $4 = "ogg" || $4 = "ape" || $4 = "flac" || $4 = "wv" ]]; then
		converttotracks "$1" "$2" "$3" "$4"
	elif [[ $4 = "mp3cue" || $4 = "mpccue" || $4 = "oggcue" || $4 = "apecue" || $4 = "flaccue" || $4 = "wvcue" ]]; then
	  M=$(echo $4 | sed 's/cue//g')
		converttocue "$1" "$2" "$3" "$M"
	elif [[ $4 = "burn" ]]; then
		converttocd "$1" "$2" "$3"
	elif [[ $4 = "extract" ]]; then
		echo ""
		echo "Release uncompressed successfully to folder $EXTRACT_FOLDER"
		echo ""
	else
		usage
		exit 1
	fi
}
##############################################################################
# checkinputformat function
#
# It will check the audio format used to compress the release, and set the EXT
# variable accordingly.
##############################################################################
checkinputformat ()
{
	if ls *.ape &>/dev/null; then
		EXT=ape
	fi
	if ls *.flac &>/dev/null; then
		EXT=flac
	fi
	if ls *.wv &>/dev/null; then
		EXT=wv
	fi
	if ls *.wav &>/dev/null; then
		EXT=wav
	fi
}

##############################################################################
# extractRelease function
#
# It will extract a relase file to the specified folder
#
# First parameter should be the release to extract
# Second parameter should be the destination folder to extract the release
##############################################################################
extractRelease ()
{
	DECOMPRESS_OK="false"
	echo "Uncompressing Release..."
	unrar x -ep "$1" "$2" &> /dev/null #Try to uncompress as RAR file
	if [ "$?" -ne "0" ] ; then
		unzip -o -j "$1" -d "$2" &> /dev/null #If unrar fails, try to unzip the file
		if [ "$?" -ne "0" ] ; then
			unace e "$1" "$2" &> /dev/null #If unzip also fails, try to unace the file
			if [ "$?" -ne "0" ] ; then
				error "Failed to uncompress the release. The file might be corrupted, or is not a valid RAR, ZIP or ACE file." 0
			else
				DECOMPRESS_OK="true"
			fi
		else
			DECOMPRESS_OK="true"
		fi
	else
		DECOMPRESS_OK="true"
	fi

	if [ $DECOMPRESS_OK = "true" ]; then
		echo ""
		echo "Successfully extracted the release to folder $2"
		echo ""
	fi
}

##############################################################################
# extractReleaseFullPath function
#
# It will extract a relase file to the specified folder
#
# First parameter should be the release to extract
# Second parameter should be the destination folder to extract the release
##############################################################################
extractReleaseFullPath ()
{
	DECOMPRESS_OK="false"
	echo "Uncompressing Release..."
	unrar x "$1" "$2" &> /dev/null #Try to uncompress as RAR file
	if [ "$?" -ne "0" ] ; then
		unzip -o "$1" -d "$2" &> /dev/null #If unrar fails, try to unzip the file
		if [ "$?" -ne "0" ] ; then
			unace x "$1" "$2" &> /dev/null #If unzip also fails, try to unace the file
			if [ "$?" -ne "0" ] ; then
				error "Failed to uncompress the release. The file might be corrupted, or is not a valid RAR, ZIP or ACE file." 0
			else
				DECOMPRESS_OK="true"
			fi
		else
			DECOMPRESS_OK="true"
		fi
	else
		DECOMPRESS_OK="true"
	fi

	if [ $DECOMPRESS_OK = "true" ]; then
		echo ""
		echo "Successfully extracted the release to folder $2"
		echo ""
	fi
}

##############################################################################
# createDefaultPreferences function
#
# It will create a preferences file based on a fixed template
##############################################################################
createDefaultPreferences ()
{
	echo "#User variables for audiommt" > $SETTINGS_FILE
	echo ""	>> $SETTINGS_FILE
	echo "#Preset used by lame to encode the MP3. Change to the one you like." >> $SETTINGS_FILE
	echo "LAME_PRESET=\"--preset standard\"" >> $SETTINGS_FILE
	echo ""	>> $SETTINGS_FILE
	echo "#Preset used by mppenc to encode the MPC. Change to the one you like." >> $SETTINGS_FILE
	echo "MPPENC_PRESET=\"--extreme\"" >> $SETTINGS_FILE
	echo "" >> $SETTINGS_FILE
	echo "#Preset used by oggenc to encode the OGG. Change to the one you like." >> $SETTINGS_FILE
	echo "OGGENC_PRESET=\"-q 5\"" >> $SETTINGS_FILE
	echo "" >> $SETTINGS_FILE
	echo "#Preset used by mac to encode the APE. Change to the one you like." >> $SETTINGS_FILE
	echo "MAC_PRESET=\"-c3000\"" >> $SETTINGS_FILE
	echo "" >> $SETTINGS_FILE
	echo "#Preset used by flac to encode the FLAC. Change to the one you like." >> $SETTINGS_FILE
	echo "FLAC_PRESET=\"-6\"" >> $SETTINGS_FILE
	echo "" >> $SETTINGS_FILE
	echo "#Preset used by wavpack to encode the WV. Change to the one you like." >> $SETTINGS_FILE
	echo "WAVPACK_PRESET=\"-h\"" >> $SETTINGS_FILE
	echo "" >> $SETTINGS_FILE
	echo "# Folders used. It is highly suggested not to change them unless you have" >> $SETTINGS_FILE
	echo "# a strong reason to do so. If you change them, please notice:" >> $SETTINGS_FILE
	echo "# 1. DEST_FOLDER _must_ be different from TEMP_FOLDER" >> $SETTINGS_FILE
	echo "# 2. TEMP_FOLDER _must_ be used exclusively by audiommt.sh, since it is" >> $SETTINGS_FILE
	echo "#    deleted completely every time the program runs. Do not, under any " >> $SETTINGS_FILE
	echo "#    circumstances, set TEMP_FOLDER to be $HOME, since all of its" >> $SETTINGS_FILE
	echo "#    contents will be deleted on exit without asking. You've been warned." >> $SETTINGS_FILE
	echo "# 3. EXTRACT_FOLDER is where the releases will be unpacked when using the" >> $SETTINGS_FILE
	echo "#    -extract switch." >> $SETTINGS_FILE
	echo "DEST_FOLDER=\"${HOME}/music\"" >> $SETTINGS_FILE
	echo "TEMP_FOLDER=\"${HOME}/mmtemp\"" >> $SETTINGS_FILE
	echo "EXTRACT_FOLDER=\"${HOME}/Desktop\"" >> $SETTINGS_FILE
	echo "Preferences file $SETTINGS_FILE created. Please check it and edit it to suit your taste."
}

##############################################################################
# lowercaseFiles function
#
# It will change all the uppercase letters found in the filenames extensionsto 
# lowercase. This is done for every file in the folder.
##############################################################################
lowercaseFiles ()
{
	for FILE in *
	do
		OLDNAME=`echo "$FILE" | sed 's/\(.*\)\..*/\1/'`
		OLDEXT=`echo "$FILE" | sed 's/.*\.//'`
		NEWEXT=`echo "$OLDEXT" | tr "[A-Z]" "[a-z]"`
		NEWNAME=$OLDNAME.$NEWEXT
		mv "$FILE" "$NEWNAME" 2>/dev/null
	done
}

##############################################################################
# error function
#
# It is used to report in a printed way the errors that occur during the
# execution of the program
#
# First parameter should be the string to print as error
# Second parameter should be the error code to be returned in case the error is
#    fatal and program should be terminated. If error code is 0, then program
#    will continue to run. Any other value (upto 255) and the program will 
#    finish returning that value
##############################################################################
error ()
{
	echo -en '\E[31m'
	echo -en '\E[5m'

	let "ERRORS += 1"

	if [ "$2" -ne "0" ]; then
		echo ""
		echo "FATAL ERROR: $1"
		echo ""
		echo -en "\E[?25h"
		echo -en "\E[0m"

		return $2
	else
		echo ""
		echo "ERROR: $1"
		echo ""
	fi

	echo -en "\E[?25h"
	echo -en "\E[0m"
}

##############################################################################
# main function
#
# It will convert the given file into the selected mode
#
# First parameter should be the mode to process the file
# Second parameter is the file or folder to process
##############################################################################
main ()
{
	# Go to the initial folder
	cd "$INIT_FOLDER"

	#Check if the release file/directory can be read. If not, exit with error.
	if [ ! -r "$2" ]; then
		error "Error: File/directory $2 is not readable or does not exist." 0
	else

		#Automatically detect the release's year based on the filename, or set it to 0 if it can't be autodetected
		YEAR=`echo "$2" | sed 's/[0-9][0-9][0-9][0-9]/<<&>>/' | sed 's/^.*<<//' | sed 's/>>.*$//' | grep [0-9][0-9][0-9][0-9]`
		if [ "x$YEAR" = "x" ]; then
			YEAR="0"
		else
			ALBUM_PREFIX="($YEAR) "
		fi

		#Delete current temp folder (if exists) and create a new one
		rm -rf "$TEMP_FOLDER" &>/dev/null
		mkdir -p "$TEMP_FOLDER" &>/dev/null

		#Check if the release is a file or a folder. If folder, do not uncompress.
		if [ -d "$2" ]; then
			DIRECTORY="true"
		fi

		#Check if parameter is a file or a folder. If file, uncompress it.
		if [[ "$DIRECTORY" = "false" && ! $1 = "extract" ]]; then # Not a folder 
			extractRelease "$2" "$TEMP_FOLDER"
		elif [[ "$DIRECTORY" = "false" && $1 = "extract" ]]; then
			A_TEMP="$(basename "$2")" # Pick the filename of the release
			B_TEMP="$(echo $A_TEMP | sed 's/.[A-Za-z][A-Za-z][A-Za-z]$//')" # Get rid of the filename extension
			C_TEMP="$EXTRACT_FOLDER/$B_TEMP" # Append the stripped filename to the extract folder. It's where we'll uncompress the release.
			echo -n "Creating extraction folder $C_TEMP"
			mkdir -p "$C_TEMP"
			extractReleaseFullPath "$2" "$C_TEMP"
			#exit 0
		else
			#It should be an already-uncompressed MMT release, since the parameter is a folder
			echo "Copying release source files to temp folder for processing..."
			cp -Rvf "$2"/* "$TEMP_FOLDER"
			DECOMPRESS_OK="true"
		fi

		#Only process files if the extraction was successful.
		if [[ "$DECOMPRESS_OK" = "true" ]]; then

			#change to working dir
			cd "$TEMP_FOLDER"
	
			# Lowercase the filenames of audio files. This avoids many issues with 
			# lower/uppercase, but it might create other issues. Let's see how it behaves...
			lowercaseFiles
	
			#Now check what kind of release this is.
			releasetype
			TYPE=$?

			#Check the format used to compress the release and set the EXT variable accordingly
			checkinputformat
		
			#Now comes the processing of the files. It will be done this way:
	
			#If the release is type 1 or 2, then process the audio IMAGE with the CUE
			#in the folder. In the case of releases type 2, any CUE will do, since they
			#should be the same, just different filename.
			if [[ $TYPE -eq 1 || $TYPE -eq 2 ]]; then
	
				#Rename audio image file and get audio file extension
				#mv *.$EXT cdimage.$EXT &>/dev/null

				#Get image filename
				I=`ls *.$EXT`

				#if there are several cue files, get just one, and ignore the rest
				i=0
				for CUE in *.cue
				do
					if [ $i -eq 0 ]; then
						#mv "$CUE" cdimage.cue &>/dev/null
						C="$CUE"
						i=1
					else
						rm -f "$CUE"
					fi
				done
		
				#convert the image
				process "$C" "${I/.$EXT/}" "$EXT" "$MODE"
			fi
	
			#If the release is type 3 or 4, it should be asked for each IMAGE which one 
			#is its CUE file, then process them in pairs. It could be guessed from the 
			#filename...but I'd rather not :D
			if [[ $TYPE -eq 3 || $TYPE -eq 4 ]]; then
				for IMAGEFILE in *.$EXT
				do
					echo ""
					echo "***********************************************************************"
					echo "Please select the number of cue associated to the following image file:"
					echo " "
					echo "--> $IMAGEFILE"
					echo " "
					echo "If there is more than one CUE that fits, just choose any of them"
					echo ""
					select CUEFILE in *.cue "Ignore this file"
					do
						if [ "$CUEFILE" = "Ignore this file" ]; then
							echo "File $IMAGEFILE ignored, it will not be processed"
						else
							process "$CUEFILE" "${IMAGEFILE/.$EXT/}" $EXT $MODE
						fi
						break
					done
				done
			fi
	
			#If the release is type 5, and it should be converted to tracks, then each audio 
			#file will be converted  with the same original name it had. No tagging
			#will be performed, since there is no way to know how the file is named, or if
			#the file name has any relation to the tags. Notice that at this moment tags from 
			#the original file are not imported, that could be done in the future.
			if [[ $TYPE -eq 5 ]]; then
				if [[ $1 = "mp3" || $1 = "mpc" || $1 = "ape" || $1 = "flac" || $1 = "wv" || $1 = "ogg" ]]; then
					ALBUM_TITLE="Unknown Album"
					ALBUM_ARTIST="Unknown Artist"
					ALBUM_SUBDIR="$ALBUM_PREFIX$ALBUM_TITLE"
					mkdir -p "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR" &>/dev/null
					for FILE in *.$EXT
					do
						shnconv "$FILE" -o wav
						rm -f "$FILE"
						compress "${FILE/.$EXT/.wav}" "$DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR/${FILE/.$EXT/.$MODE}" $MODE
					done
					echo ""
					echo "Done. The resultant files are in $DEST_FOLDER/$ALBUM_ARTIST/$ALBUM_SUBDIR folder, but they have no tags."
					echo ""
				else
					#If the release is type 5, and it should be converted to CD, then an image 
					#and a cue files should be created from the audio files found. They will be 
					#arranged in alphabetical order in the final image, since they are usually 
					#numbered 01*,02* etc. Rename them to match the order you want them to be 
					#in the CD.
					shncue -r ascii *.$EXT > joined.cue
					shnjoin -r ascii *.$EXT
					rm -f *.$EXT
					process joined.cue joined wav $MODE
				fi
			fi
	
			#If the release is type 6, it will be ignored, since we have no idea what 
			#it might be. Maybe it's some other RAR file or something like that.
			if [[ $TYPE -eq 6 ]]; then
				error "This file or folder does not seem to contain a valid MMT release. Ignoring it." 0
			fi
	
			#Move the extra files of the release (scanned images, log, txt, etc)
			#to the output folder. This is not done if the release is burned to CD.
			if [[ ! $1 = "burn" && ! $TYPE -eq 6 ]]; then
				moveExtraFiles
			fi
	
			#Finally, delete any files in the temp folder and the temp folder itself
			cleanup
		fi
	fi
}

##############################################################################
# Main program
##############################################################################

#Set a global error variable
ERRORS=0

#Try to detect the required tools, and issue an error if a required one is missing
#or a warning if an optional one is missing
REQTOOLS="sed seq bash flac unrar wvunpack mac cuebreakpoints shnsplit lame id3 vorbiscomment metaflac"
REQCOMPLETE=1
OPTTOOLS="cdrecord unzip unace"
for command in $REQTOOLS; do
	which $command &> /dev/null
	if [ ! $? = 0 ]; then
		echo "Error, command "$command" not found. Install it to continue."
		REQCOMPLETE=0
	fi
done
if [ $REQCOMPLETE = 0 ]; then
	exit
fi

#Start the program by setting the options file
SETTINGS_FILE="${HOME}/.audiommtrc"

#Set the conversion mode to none
MODE="none"

#By default, no extension in the input audio files is assumed
EXT=""

#Assume the release is compressed
DIRECTORY="false"

#Prefix of the destination subdirectory
ALBUM_PREFIX=""
DECOMPRESS_OK="false"

#Check for the preferences file, and create a default if there is none.
if [ -r $SETTINGS_FILE ]; then
	source $SETTINGS_FILE
else
	createDefaultPreferences
	exit 2
fi

#Check the mode and set it as global variable
if [[ "$1" = "-mp3" || "$1" = "-mpc" || "$1" = "-ogg" || "$1" = "-ape" || 
      "$1" = "-flac" || "$1" = "-wv" || "$1" = "-mp3cue" || "$1" = "-mpccue" || 
      "$1" = "-oggcue" || "$1" = "-apecue" || "$1" = "-flaccue" || "$1" = "-wvcue" || 
      "$1" = "-burn" || "$1" = "-extract" ]]; then
	MODE=$(echo $1 | sed 's/-//g')
else
	usage
	exit 3
fi     

#Set the output folder according to the output file type
DEST_FOLDER="$DEST_FOLDER/$MODE"

# Set the current folder as initial folder
INIT_FOLDER=$(pwd)

# Drop the mode from the parameters list
shift

# Loop over the rest of parameters until there are no more.
# All the remaining parameters should be files to convert.
while [[ ! x"$1" = x"" ]]; do
	echo ""
	echo "   *** Starting processing of file "$1" *** "
	echo ""
	main $MODE "$1"
	echo ""
	echo "   *** Finished processing of file "$1" *** "
	echo ""
	shift
done

#Report total number of errors
if [[ ! x"$ERRORS" = x"0" ]]; then
	echo "There were $ERRORS errors. Check the output in your console to find them"
else
	echo "All files processed successfully"
fi
echo ""

#Exit normally.
exit 0

